module GeolocationService::Services
  class BaseService
    extend Dry::Initializer
    extend Dry::Monads::Try::Mixin
    include Dry::Monads::Result::Mixin

    def self.call(**args)
      validate(contract, args).bind do |contract|
        Try() do
          self.new(**contract.to_h).call
        end.to_result
      end
    end

    def self.validate(contract, args)
      raise ArgumentError.new("missing contract or args") if contract.nil? || args.nil?

      contract.call(args.to_h).to_monad
    rescue => e
      Dry::Monads::Failure(e)
    end

    def self.contract
      unless defined?(self::Contract)
        raise NotImplementedError.new("#{self.class.name} - doesnt have Contract")
      end

      self::Contract.new
    end

    def call
      raise NotImplementedError
    end
  end
end
