module GeolocationService::Structs
  IpStruct = Struct.new(:id, :address, :mystery_value) do
    def to_sql(created_at)
      if mystery_value.blank?
        "(#{id}, '#{address}', NULL, '#{created_at}', '#{created_at}')"
      else
        "(#{id}, '#{address}', '#{mystery_value}', '#{created_at}', '#{created_at}')"
      end
    end

    def self.insert_sql(ips)
      created_at = DateTime.now.to_s(:db)
      records = ips.map { |ip| ip.to_sql(created_at) }

      "INSERT INTO ips (id, address, mystery_value, created_at, updated_at) VALUES #{records.join(', ')}"
    end
  end

  CityStruct = Struct.new(:id, :name, :country_id) do
    def to_sql(created_at)
      if country_id.blank?
        "(#{id}, \"#{name}\", NULL, '#{created_at}', '#{created_at}')"
      else
        "(#{id}, \"#{name}\", '#{country_id}', '#{created_at}', '#{created_at}')"
      end
    end

    def self.insert_sql(cities)
      created_at = DateTime.now.to_s(:db)
      records = cities.map { |city| city.to_sql(created_at) }

      "INSERT INTO cities (id, name, country_id, created_at, updated_at) VALUES #{records.join(', ')}"
    end
  end

  CountryStruct = Struct.new(:id, :code, :name) do
    def to_sql(created_at)
      if name.blank?
        "(#{id}, '#{code}', NULL, '#{created_at}', '#{created_at}')"
      else
        "(#{id}, '#{code}', \"#{name}\", '#{created_at}', '#{created_at}')"
      end
    end

    def self.insert_sql(countries)
      created_at = DateTime.now.to_s(:db)
      records = countries.map { |country| country.to_sql(created_at) }

      "INSERT INTO countries (id, code, name, created_at, updated_at) VALUES #{records.join(', ')}"
    end
  end

  LocationStruct = Struct.new(:ip_id, :latitude, :longitude, :city_id) do
    def to_sql(created_at)
      sql = [ ip_id, latitude, longitude, city_id ]
        .map { |v| v.presence || 'NULL' }
        .join(', ')

      "(#{sql}, '#{created_at}', '#{created_at}')"
    end

    def self.insert_sql(locations)
      created_at = DateTime.now.to_s(:db)
      records = locations.map { |location| location.to_sql(created_at) }

      "INSERT INTO locations (ip_id, latitude, longitude, city_id, created_at, updated_at) VALUES #{records.join(', ')}"
    end
  end
end
